/**
 * Watches for when a table header scrolls out of view.
 * When it does, adds a cloned header that is fixed at the top of the scrolling element.
 */
var StickyTableHeader = /** @class */ (function () {
    /**
     * Creates the instance, and begins listening for scroll events.
     * @param table The table to add sticky headers to.
     * @param fixedElement A fixed element at the top of the screen to use as an anchor, rather than the top of the screen itself.
     */
    function StickyTableHeader(table, fixedElement) {
        var _this = this;
        this.table = table;
        this.fixedElement = fixedElement;
        this.domObserver = new MutationObserver(function (mutations) {
            _this.syncColumnWidths();
        });
        this.originalWidths = [];
        this.headerHeight = 0;
        if (!table.tHead) {
            throw Error("Table must have a header section.");
        }
        this.header = table.tHead;
        this.scrollElement = StickyTableHeader.getParentScrollElement(table);
        if (this.scrollElement) {
            if (this.scrollElement.style.height || this.scrollElement.style.maxHeight) {
                this.relativeParent = this.scrollElement;
            }
        }
        this.handleScroll = this.handleScroll.bind(this);
        this.onPossibleContentChange = this.onPossibleContentChange.bind(this);
        document.addEventListener("scroll", this.handleScroll);
        if (this.scrollElement) {
            this.scrollElement.addEventListener("scroll", this.handleScroll);
        }
        table.addEventListener("keydown", this.onPossibleContentChange);
        table.addEventListener("mouseup", this.onPossibleContentChange);
    }
    StickyTableHeader.prototype.handleScroll = function () {
        // If the grid is in an element with a scrollbar, then the header should stick to the top of that element,
        //  otherwise if there is a fixed (non scrolling) element at the top of the page, then the headers must appear below that.
        //  otherwise the headers will stick to the top of the screen.
        var fixedElement = null;
        if (!this.relativeParent && this.fixedElement && getComputedStyle(this.fixedElement).position === "fixed") {
            fixedElement = this.fixedElement;
        }
        var topBoundary = this.relativeParent ? this.relativeParent.getBoundingClientRect().top : (fixedElement ? fixedElement.getBoundingClientRect().bottom : 0);
        var tableBounds = this.table.getBoundingClientRect();
        var tbodyBounds = this.table.tBodies[this.table.tBodies.length - 1].getBoundingClientRect();
        if (tableBounds.top < topBoundary && tbodyBounds.bottom - this.headerHeight > topBoundary) {
            if (!this.headerCopy) {
                // Clone the header, and re-insert it into the table. The table will then have 2 headers.
                // The original headers position is set to "fixed" which makes it float.
                // The original header is floated instead of the clone because the react events are attached to these cells, and they are the cells that are visible. Without this, sorting wouldn't work.
                // The column widths of the floating header then need to be synchronized with the header that is still inside the table layout.
                this.headerHeight = this.header.getBoundingClientRect().height;
                this.headerCopy = this.header.cloneNode(true);
                this.table.insertBefore(this.headerCopy, this.header);
                this.header.classList.add("neo-grid-fixed-header");
                this.storeColumnWidths();
                this.syncColumnWidths();
                this.domObserver.observe(this.header, { childList: true, subtree: true, attributes: false });
            }
            this.header.style.left = tableBounds.left + "px";
            this.header.style.top = topBoundary + "px";
            if (this.scrollElement) {
                var left = this.scrollElement.scrollLeft;
                var width = this.scrollElement.clientWidth + left - 1;
                this.header.style.clipPath = "rect(0px ".concat(width, "px 100% ").concat(left, "px)");
            }
        }
        else {
            this.removeCopy();
        }
    };
    StickyTableHeader.prototype.removeCopy = function () {
        if (this.headerCopy) {
            this.domObserver.disconnect();
            this.restoreColumnWidths();
            this.header.classList.remove("neo-grid-fixed-header");
            this.header.style.clipPath = "";
            this.header.style.left = "";
            this.header.style.top = "";
            this.table.removeChild(this.headerCopy);
            this.headerCopy = undefined;
        }
    };
    StickyTableHeader.prototype.onPossibleContentChange = function () {
        var _this = this;
        setTimeout(function () { return _this.syncColumnWidths(); }, 0);
    };
    StickyTableHeader.prototype.syncColumnWidths = function () {
        if (this.headerCopy) {
            for (var i = 0; i < this.header.rows.length; i++) {
                var originalRow = this.header.rows[i], copyRow = this.headerCopy.rows[i];
                if (originalRow.cells.length !== copyRow.cells.length) {
                    this.removeCopy();
                    this.handleScroll();
                    return;
                }
                for (var j = 0; j < originalRow.cells.length; j++) {
                    var originalCell = originalRow.cells[j], copyCell = copyRow.cells[j];
                    originalCell.style.width = window.getComputedStyle(copyCell).width;
                    originalCell.style.minWidth = originalCell.style.width;
                    originalCell.style.maxWidth = originalCell.style.width;
                }
            }
        }
    };
    StickyTableHeader.prototype.storeColumnWidths = function () {
        this.originalWidths = [];
        for (var i = 0; i < this.header.rows.length; i++) {
            var row = this.header.rows[i];
            for (var j = 0; j < row.cells.length; j++) {
                var cell = row.cells[j];
                this.originalWidths.push({ cell: cell, width: cell.style.width });
            }
        }
    };
    StickyTableHeader.prototype.restoreColumnWidths = function () {
        for (var _i = 0, _a = this.originalWidths; _i < _a.length; _i++) {
            var cellRef = _a[_i];
            cellRef.cell.style.width = cellRef.width;
        }
    };
    StickyTableHeader.prototype.dispose = function () {
        document.removeEventListener("scroll", this.handleScroll);
        if (this.scrollElement) {
            this.scrollElement.removeEventListener("scroll", this.handleScroll);
        }
        this.table.removeEventListener("keydown", this.onPossibleContentChange);
        this.table.removeEventListener("mouseup", this.onPossibleContentChange);
        this.removeCopy();
    };
    /**
     * Returns the first parent element that has a scrollbar.
     */
    StickyTableHeader.getParentScrollElement = function (element) {
        var current = element.parentElement;
        var depth = 0;
        while (current && depth < 5) {
            var computedStyle = window.getComputedStyle(current);
            if (computedStyle.overflowX === "auto" || computedStyle.overflowX === "scroll" || computedStyle.overflowY === "auto" || computedStyle.overflowY === "scroll") {
                return current;
            }
            current = current.parentElement;
        }
        return undefined;
    };
    return StickyTableHeader;
}());
export { StickyTableHeader };
