import { __awaiter, __decorate, __generator, __metadata } from "tslib";
import { Components } from "@singularsystems/neo-core";
import { Transition } from "./Transition";
import { makeObservable, observable } from "mobx";
var TransitionController = /** @class */ (function () {
    /**
     * Creates a transition controller.
     * @param initialName Name of the panel to display initially. Not required if `isVisible` props are defined on the panels.
     * @param transition Transition type to use by default on all panels.
     * @param backTransition Transition type to use when going backwards.
     */
    function TransitionController(initialName, transition, backTransition) {
        this._selectedName = "";
        this.direction = 1;
        this.isFirstTransition = true;
        this.isTransitioning = false;
        /**
         * @internal
         */
        this.wasAutoCreated = false;
        this.panels = [];
        this.panelStates = [];
        this._selectedName = initialName !== null && initialName !== void 0 ? initialName : "";
        makeObservable(this);
        this.defaultTransition = transition !== null && transition !== void 0 ? transition : TransitionController.transition;
        this.defaultBackTransition = backTransition !== null && backTransition !== void 0 ? backTransition : this.defaultTransition;
    }
    Object.defineProperty(TransitionController.prototype, "selectedPanel", {
        /**
         * Returns the name of the selected panel.
         */
        get: function () {
            return this._selectedName;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Begins a transition to the specified panel.
     * @param waitFor A promise on which to wait before showing the to panel. Use if data needs to be loaded.
     */
    TransitionController.prototype.transition = function (toPanelName, waitFor) {
        if (!this.isTransitioning) {
            var currentIndex = this.panels.indexOf(this._selectedName);
            var newIndex = this.panels.indexOf(toPanelName);
            this.doTransition(toPanelName, newIndex - currentIndex, waitFor);
        }
    };
    TransitionController.prototype.forward = function (toPanelName, waitFor) {
        this.doTransition(toPanelName !== null && toPanelName !== void 0 ? toPanelName : "", 1, waitFor);
    };
    TransitionController.prototype.back = function (toPanelName, waitFor) {
        this.doTransition(toPanelName !== null && toPanelName !== void 0 ? toPanelName : "", -1, waitFor);
    };
    /**
     * @internal
     * Called by the container on mount.
     */
    TransitionController.prototype.registerContainer = function (element) {
        this.containerElement = element;
    };
    /**
     * @internal
     * Called by the container on unmount.
     */
    TransitionController.prototype.disposeContainer = function () {
        this.containerElement = undefined;
    };
    /**
     * @internal
     * Called by each panel on mount.
     */
    TransitionController.prototype.registerPanel = function (name, hasVisibleProp) {
        if (this.wasAutoCreated && !hasVisibleProp) {
            throw "Panel with name ".concat(name, " must specify the isVisible prop, or you must pass a TransitionController to the TransitionContainer.");
        }
        if (this.panels.indexOf(name) < 0) {
            this.panels.push(name);
        }
    };
    /**
     * @internal
     * Called by each panel on unmount.
     */
    TransitionController.prototype.disposePanel = function (name) {
        this.panels.remove(name);
        var panelState = this.panelStates.find(function (c) { return c.name === name; });
        if (panelState) {
            clearTimeout(panelState.inTimer);
            clearTimeout(panelState.outTimer);
        }
    };
    /**
     * @internal
     * Called by each panel after render.
     */
    TransitionController.prototype.afterUpdate = function (name, element, transition, animateHeight) {
        if (this._selectedName === name && !this.isTransitioning) {
            var panelState_1 = this.panelStates.find(function (c) { return c.name === name; });
            var transitionIn = false;
            if (!panelState_1) {
                panelState_1 = { name: name, element: element, transition: transition };
                this.panelStates.push(panelState_1);
                transitionIn = true;
            }
            if (panelState_1.outTimer) {
                clearTimeout(panelState_1.outTimer);
                panelState_1.outTimer = undefined;
                transitionIn = true;
            }
            if (transitionIn && !this.isFirstTransition) {
                panelState_1.inTimer = this.transitionElement(panelState_1.element, panelState_1.transition, this.direction > 0, false, animateHeight !== null && animateHeight !== void 0 ? animateHeight : true, function () {
                    panelState_1.inTimer = undefined;
                });
            }
            this.isFirstTransition = false;
        }
    };
    /**
     * @internal
     * If the panel is selected, or is busy transitioning.
     */
    TransitionController.prototype.isPanelVisible = function (name) {
        return this._selectedName === name ||
            this.panelStates.find(function (c) { return c.name === name; }) !== undefined;
    };
    TransitionController.prototype.doTransition = function (toName, direction, waitFor) {
        return __awaiter(this, void 0, void 0, function () {
            var e_1, oldKey;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this._selectedName !== toName)) return [3 /*break*/, 5];
                        this.direction = direction;
                        this.isTransitioning = true;
                        this.beginRemove();
                        if (!waitFor) return [3 /*break*/, 4];
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, waitFor];
                    case 2:
                        _a.sent();
                        return [3 /*break*/, 4];
                    case 3:
                        e_1 = _a.sent();
                        this.isTransitioning = false;
                        oldKey = this._selectedName;
                        this._selectedName = "";
                        this._selectedName = oldKey;
                        return [3 /*break*/, 4];
                    case 4:
                        // This will cause the panel to render, and call registerPanel().
                        this._selectedName = toName;
                        this.isTransitioning = false;
                        _a.label = 5;
                    case 5: return [2 /*return*/];
                }
            });
        });
    };
    TransitionController.prototype.beginRemove = function () {
        var _this = this;
        var _loop_1 = function (panelState) {
            if (panelState.inTimer) {
                clearTimeout(panelState.inTimer);
                panelState.inTimer = undefined;
            }
            if (!panelState.outTimer) {
                panelState.outTimer = this_1.transitionElement(panelState.element, panelState.transition, this_1.direction > 0, true, true, function () {
                    _this.panelStates.remove(panelState);
                });
            }
        };
        var this_1 = this;
        for (var _i = 0, _a = this.panelStates; _i < _a.length; _i++) {
            var panelState = _a[_i];
            _loop_1(panelState);
        }
    };
    TransitionController.prototype.transitionElement = function (element, transition, toLeft, out, animateHeight, onComplete) {
        var _this = this;
        var canAnimate = !TransitionController.disableTransitions;
        transition !== null && transition !== void 0 ? transition : (transition = toLeft ? this.defaultTransition : this.defaultBackTransition);
        if (canAnimate) {
            element.style.overflow = "hidden";
            var elementHeight_1 = element.getBoundingClientRect().height;
            if (this.containerElement && !this.containerElement.classList.contains("transitioning")) {
                this.containerElement.style.height = elementHeight_1 + "px";
                this.containerElement.classList.add("transitioning");
            }
            element.style.opacity = out ? "1" : "0";
            transition.setBeforeStyles(element, out, toLeft);
            Components.AnimationHelper.triggerReflow(element);
            element.classList.add("animate-".concat(out ? "out" : "in"));
            Components.AnimationHelper.triggerReflow(element);
            element.style.opacity = out ? "0" : "1";
            transition.setAfterStyles(element, out, toLeft);
            // Timeout needed to give time for child elements to render, in order to correctly calculate height.
            setTimeout(function () {
                if (!out && _this.containerElement) {
                    _this.containerElement.style.height = animateHeight ? (transition.changesHeight ? elementHeight_1 : element.getBoundingClientRect().height) + "px" : "";
                }
            }, 0);
        }
        var cleanup = function () {
            if (out) {
                // Element will be removed from the dom, no need to cleanup props.
            }
            else {
                element.classList.remove("animate-in");
                element.style.opacity = "";
                element.style.overflow = "";
                transition.cleanup(element);
            }
            onComplete();
            if (_this.containerElement && !_this.panelStates.some(function (c) { return c.inTimer; })) {
                _this.containerElement.classList.remove("transitioning");
                if (_this.panelStates.length > 0) {
                    _this.containerElement.style.height = "";
                }
            }
        };
        if (canAnimate) {
            var transitionDuration = parseFloat(window.getComputedStyle(element).transitionDuration) * 1000;
            return setTimeout(cleanup, transitionDuration);
        }
        else {
            cleanup();
            return null;
        }
    };
    /** Global default transition. */
    TransitionController.transition = Transition.leftRight;
    /** Allows transitions to be disabled for the whole app. */
    TransitionController.disableTransitions = false;
    __decorate([
        observable.ref,
        __metadata("design:type", String)
    ], TransitionController.prototype, "_selectedName", void 0);
    __decorate([
        observable.shallow,
        __metadata("design:type", Array)
    ], TransitionController.prototype, "panelStates", void 0);
    return TransitionController;
}());
export { TransitionController };
