import { __awaiter, __decorate, __extends, __generator, __metadata } from "tslib";
import React from 'react';
import { makeObservable, observable, runInAction } from 'mobx';
import { Misc } from '@singularsystems/neo-core';
import { ViewParameterList } from '../Routing/ViewParameterList';
import { ViewState } from './ViewState';
import { BreadCrumbItem } from '../Routing/BreadCrumbItem';
import { NeoReactTypes } from '../Modules/Types';
import ViewModelBase from './ViewModelBase';
var ViewBase = /** @class */ (function (_super) {
    __extends(ViewBase, _super);
    /**
     * Creates a view
     * @param props React props.
     * @param viewName Name of the view.
     * @param viewModel Instance of view model, or view model type, or callback which returns a view model.
     */
    function ViewBase(viewName, viewModel, props) {
        var _this = _super.call(this, props) || this;
        _this.viewName = "";
        /** Indicates that the view requests to be displayed full screen. Up to the main layout to accommodate this. */
        _this.fullScreen = false;
        /** Indicates that the VM has initialised, and viewParamsUpdated() has been called. */
        _this.hasInitialised = false;
        _this.routingState = Misc.Globals.appService.get(NeoReactTypes.Routing.GlobalRoutingState);
        _this.navigation = Misc.Globals.appService.get(NeoReactTypes.Routing.NavigationHelper);
        if (typeof viewName !== "string") {
            throw "Error: When inheriting ViewBase, you must pass the view name into the base constructor.";
        }
        _this.viewName = viewName;
        _this.viewState = _this.getStoredViewState(viewModel);
        _this.routingState.currentView = _this;
        makeObservable(_this);
        return _this;
    }
    Object.defineProperty(ViewBase, "currentView", {
        // For backwards compatibility
        get: function () {
            return Misc.Globals.appService.get(NeoReactTypes.Routing.GlobalRoutingState).currentView;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewBase.prototype, "viewModel", {
        get: function () {
            return this.viewState.viewModel;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewBase.prototype, "taskRunner", {
        get: function () {
            return this.viewModel.taskRunner;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ViewBase.prototype, "viewParams", {
        get: function () {
            return this.viewState.viewParams;
        },
        enumerable: false,
        configurable: true
    });
    ViewBase.prototype.getStoredViewState = function (viewModel) {
        return this.createViewState(viewModel);
    };
    ViewBase.prototype.createViewState = function (viewModel) {
        var params = this.constructor.params;
        var paramsList = new ViewParameterList(params, this);
        var viewModelInstance = ViewModelBase.create(viewModel);
        return new ViewState(paramsList, viewModelInstance);
    };
    ViewBase.prototype.componentDidMount = function () {
        return __awaiter(this, void 0, void 0, function () {
            var paramsUpdated;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        paramsUpdated = this.routingState.currentRoute ? (this.viewState.viewParams).update(this.routingState.currentRoute) : false;
                        if (!!this.viewState.hasSetup) return [3 /*break*/, 2];
                        this.viewState.hasSetup = true;
                        return [4 /*yield*/, this.initialise()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!paramsUpdated) return [3 /*break*/, 4];
                        return [4 /*yield*/, this.viewParamsUpdated()];
                    case 3:
                        _a.sent();
                        _a.label = 4;
                    case 4:
                        this.hasInitialised = true;
                        return [2 /*return*/];
                }
            });
        });
    };
    ViewBase.prototype.componentWillUnmount = function () {
        this.disposeViewModel();
    };
    ViewBase.prototype.updateViewParams = function (match) {
        var _this = this;
        runInAction(function () { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.viewState.viewParams).update(match)) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.viewParamsUpdated()];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        }); });
    };
    /**
     * viewParamsUpdated() is called when the component is mounted, and whenever a route parameter or query parameter gets a changed value.
     */
    ViewBase.prototype.viewParamsUpdated = function () {
    };
    /** Gets the parent breadcrumbs, and the breadcrumb for this view. */
    ViewBase.prototype.getBreadCrumbStart = function () {
        var _this = this;
        var _a;
        var items = [];
        // Get parent items
        var menuRoute = (_a = this.routingState.currentRoute) === null || _a === void 0 ? void 0 : _a.route;
        var parent = menuRoute.breadCrumbParent;
        while (parent) {
            items.splice(0, 0, new BreadCrumbItem(parent.name, parent.path));
            parent = parent.breadCrumbParent;
        }
        // Add view name
        var firstParam = this.viewState.viewParams.params[0];
        items.push(new BreadCrumbItem(function () { return _this.viewName; }, (firstParam === null || firstParam === void 0 ? void 0 : firstParam.routeParameter.required) ? "" : this.navigation.getCurrentViewPath()));
        return items;
    };
    /**
     * Initialise is called from componentDidMount().
     * Will only be called once per persistent view for the lifetime of the app.
     * If overridden, you must call super.initialise() to initialise the ViewModel.
     */
    ViewBase.prototype.initialise = function () {
        return this.viewModel.initialise();
    };
    ViewBase.prototype.disposeViewModel = function () {
        this.viewModel.dispose();
    };
    ViewBase.prototype.getBreadCrumbList = function () {
        var breadCrumbList = [];
        if (!this.breadCrumbStartItems) {
            this.breadCrumbStartItems = this.getBreadCrumbStart();
        }
        breadCrumbList.push.apply(breadCrumbList, this.breadCrumbStartItems);
        breadCrumbList.push.apply(breadCrumbList, this.viewParams.getBreadCrumbList());
        return breadCrumbList;
    };
    /**
     * Called when the user navigates to a new view, or when exiting the site.
     * Return a string to show to the user in a dialog.
     */
    ViewBase.prototype.onLeave = function () {
        return undefined;
    };
    /**
     * Called whenever the route changes. Including navigating to a new view, leaving the site, or changing a route parameter in this view.
     * Note: if props.leavePrompt is set, onLeave() will not be called.
     * @param props Set leavePrompt to `false`, a `string`, or a promise from a modal.
     */
    ViewBase.prototype.beforeRouteChanged = function (props) {
    };
    __decorate([
        observable.ref,
        __metadata("design:type", String)
    ], ViewBase.prototype, "viewName", void 0);
    __decorate([
        observable.ref,
        __metadata("design:type", Boolean)
    ], ViewBase.prototype, "fullScreen", void 0);
    __decorate([
        observable.ref,
        __metadata("design:type", Boolean)
    ], ViewBase.prototype, "hasInitialised", void 0);
    return ViewBase;
}(React.Component));
export default ViewBase;
