import { __assign, __spreadArray } from "tslib";
import { MenuRoute } from './MenuRoute';
var RouteProvider = /** @class */ (function () {
    /**
     * Creates a route provider which provides a list of flattened routes.
     * @param menuRoutes Routes which appear on the main menu.
     * @param pureRoutes Routes which don't have a menu item.
     * @param notFoundComponent Component to render when the url doesn't match any of the above routes.
     */
    function RouteProvider(menuRoutes, pureRoutes, notFoundComponent) {
        this.menuRoutes = menuRoutes;
        this.pureRoutes = pureRoutes;
        this.notFoundComponent = notFoundComponent;
        this.key = 1;
        this.routeIndex = {};
        this.staticRoutes = [];
        this.parameterisedRoutes = [];
        this.allRoutes = [];
        this.flatten(this.allRoutes, menuRoutes);
        if (pureRoutes)
            this.flatten(this.allRoutes, pureRoutes);
        this.staticRoutes = this.allRoutes.filter(function (c) { return c.params.length === 0; });
        this.parameterisedRoutes = this.allRoutes
            .filter(function (c) { return c.params.length > 0; })
            .map(function (route) { return ({ route: route, order: route.path.split("/").length }); })
            .sortBy(function (c) { return c.order; }, "desc")
            .map(function (c) { return c.route; });
    }
    /** Gets the path the component is bound to. If the component is bound to multiple paths, the first path is returned. */
    RouteProvider.prototype.getPathForComponent = function (component) {
        var _a;
        return (_a = this.allRoutes.find(function (c) { return c.component === component; })) === null || _a === void 0 ? void 0 : _a.path;
    };
    /** @internal */
    RouteProvider.prototype.getRouteForPath = function (path) {
        return RouteProvider.getRouteForPath(this.staticRoutes, this.parameterisedRoutes, path);
    };
    /** @internal */
    RouteProvider.getRouteForPath = function (staticRoutes, parameterisedRoutes, path) {
        var queryIndex = path.indexOf("?");
        if (queryIndex >= 0) {
            path = path.substring(0, queryIndex);
        }
        var lcPath = path.toLowerCase();
        var found = staticRoutes.find(function (route) { return route.path === lcPath; });
        if (found) {
            return { route: found, path: path };
        }
        if (path.endsWith("/")) {
            // special case for where an extra "/" is added to the url by mistake.
            var trimmedPath_1 = lcPath.substring(0, path.length - 1);
            var found_1 = staticRoutes.find(function (route) { return route.path === trimmedPath_1; });
            if (found_1) {
                return { route: found_1, path: path };
            }
        }
        var matches = parameterisedRoutes.filter(function (route) { return lcPath.startsWith(route.path); });
        for (var _i = 0, matches_1 = matches; _i < matches_1.length; _i++) {
            var match = matches_1[_i];
            var paramString = path.substring(match.path.length);
            // Special case for root path which has its leading "/" removed by the navigation helper.
            if (match.path === "/") {
                paramString = "/" + paramString;
            }
            var parts = paramString.split("/").slice(1);
            var partCount = parts.length;
            // Allow extra "/" 
            if (partCount > 0 && parts[partCount - 1] === "") {
                partCount -= 1;
            }
            if (match.params.length >= partCount) {
                var paramValues = {};
                var isValid = true;
                for (var i = 0; i < match.params.length; i++) {
                    var param = match.params[i];
                    var value = parts[i];
                    paramValues[param.name] = value;
                    if (param.required && !value) {
                        isValid = false;
                        break;
                    }
                }
                if (isValid) {
                    return { route: match, path: path, params: paramValues };
                }
            }
        }
        return null;
    };
    RouteProvider.prototype.flatten = function (into, routes, parent) {
        var _a, _b;
        for (var _i = 0, routes_1 = routes; _i < routes_1.length; _i++) {
            var route = routes_1[_i];
            var hasComponent = route.path !== undefined && route.component;
            if (hasComponent) {
                if (parent && !route.breadCrumbParent) {
                    route.breadCrumbParent = parent;
                }
                this.processRoute(into, route);
            }
            var menuRoute = route;
            var children = __spreadArray(__spreadArray([], ((_a = route.routeChildren) !== null && _a !== void 0 ? _a : []), true), ((_b = menuRoute.children) !== null && _b !== void 0 ? _b : []), true);
            this.flatten(into, children, (hasComponent && menuRoute.showInBreadCrumb !== false) ? menuRoute : undefined);
        }
    };
    /**
     * Create a copy of the route info, and process it.
     * Adds a unique key, and adds route parameters extracted from the component static params object.
     */
    RouteProvider.prototype.processRoute = function (into, route) {
        var processed = __assign(__assign({}, route), { path: route.path, params: [], key: this.key++ });
        if (route instanceof MenuRoute) {
            processed.path = route.basePath;
        }
        processed.path = processed.path.toLowerCase();
        if (processed.path.length > 1 && processed.path.endsWith("/")) {
            processed.path = processed.path.substring(0, processed.path.length - 1);
        }
        var viewBaseComponent = route.component;
        if (viewBaseComponent.params) {
            for (var param in viewBaseComponent.params) {
                var routeParam = viewBaseComponent.params[param];
                if (!routeParam.isQuery) {
                    processed.params.push(__assign(__assign({}, routeParam), { name: param }));
                }
            }
        }
        if (!this.routeIndex[processed.path]) {
            this.routeIndex[processed.path] = true;
            into.push(processed);
        }
    };
    return RouteProvider;
}());
export default RouteProvider;
